#!/bin/bash
##
## Pipeline script for the Hisat alignment step. 
## 
##
## Environment variables that should be defined (in job.sh) before calling this script
##  -FastqFolder: Path to folder with FASTQ files
##  -HisatFolder: Folder for output. Will be created or cleared from existing files
##  -BOWTIE: Path to BowTie program
##  -RMidx: Path to reference genome used for masking
##  -BowTieOptions: Other options for the BowTie program
##  -HISAT: Path to Hisat program
##  -Tidx: Path to the reference genome used for alignment
##  -HisatOptions: Other options for the Hisat program
##  -SAMTOOLS: Path to samtools program
##  -JAVA: Path to java
##  -PICARD: Path to Picard JAR file
##  -MarkDuplicatesOptions: Options to the Picard MarkDuplicates step
##  -GATK: Path to GATK JAR file
##  -DBSNP: Path to VCF file with SNP variants that should be genotyped
##  -HCRef: Path to reference genome FASTA file
##  -HaplotypeCallerOptions: Other options for the the HaplotypeCaller program
##  -TMPDIR: Path to a local temporary working directory
##

set -e

## Import utility functions
source ./reggie-utils.sh

## Verify settings in options
rg_var_isdir "FastqFolder" "TMPDIR" "WD"
rg_var_isfile "BOWTIE" "HISAT" "SAMTOOLS" "JAVA" "PICARD" "GATK" "HCRef" "DBSNP"
rg_var_isset "RMidx" "HisatFolder"

## Move to the temporary working directory and create subdirectories
cd ${TMPDIR}
mkdir -p fastq
mkdir -p masked
mkdir -p aligned
mkdir -p done

## Copy FASTQ files to local working directory
if [ ! -f "done/copy.done" ]; then
	rg_progress 10 "Copying FASTQ files"
	cp ${FastqFolder}/*.fastq.gz fastq
	touch "done/copy.done"
fi

## Run Bowtie2 if masked/*.fastq.gz doesn't exists
if [ ! -f "done/mask.done" ]; then
	rg_progress 20 "Running Bowtie2 (${NumThreads} threads)"
	## Find R1 and R2 FASTQ files
	FASTQ1=`find fastq -name "*_R1.fastq.gz" -print -quit 2> /dev/null`
	FASTQ2=`find fastq -name "*_R2.fastq.gz" -print -quit 2> /dev/null`
	rg_var_isfile "FASTQ1" "FASTQ2"
	${WD}/stdwrap.sh ${BOWTIE} \
		-p ${NumThreads} \
		${BowTieOptions} \
		--un-conc-gz masked/R%.fastq.gz \
		-x ${RMidx} \
		-1 ${FASTQ1} \
		-2 ${FASTQ2} \
		-S /dev/null \
		> masked/masked.out
	touch "done/mask.done"
fi
	
## Run Hisat 
if [ ! -f "done/align.done" ]; then
	rg_progress 30 "Running Hisat (${NumThreads} threads)"
	${WD}/stdwrap.sh ${HISAT} \
		-p ${NumThreads} \
		${HisatOptions} \
		-x ${Tidx} \
		-1 masked/R1.fastq.gz \
		-2 masked/R2.fastq.gz \
		-S aligned/alignment.sam \
		> aligned/hisat.out
	touch "done/align.done"
fi

## Sort alignment.sam and save to alignment.bam
if [ ! -f "done/sort.done" ]; then
	rg_progress 50 "Sorting alignment file"
	${SAMTOOLS} sort -@ ${NumThreads} \
		-o aligned/alignment.bam -O bam \
		-T aligned/tmp \
		aligned/alignment.sam
	
	rm -f aligned/alignment.sam
	touch "done/sort.done"
fi


if [ ! -f "done/markduplicates.done" ]; then
	rg_progress 60 "Running picard MarkDuplicates"
	${WD}/stdwrap.sh ${JAVA} \
		-Dpicard.useLegacyParser=false ${JavaOptions} \
		-jar ${PICARD} MarkDuplicates \
		-INPUT aligned/alignment.bam \
		-OUTPUT aligned/alignment.bam.tmp_picard \
		-METRICS_FILE aligned/alignment_picardmetrics.csv \
		${MarkDuplicatesOptions} \
		> aligned/picard_MarkDuplicates.out
	mv -f aligned/alignment.bam.tmp_picard aligned/alignment.bam
	touch "done/markduplicates.done"
fi

if [ ! -f "done/index.done" ]; then
	rg_progress 80 "Creating index"
	${SAMTOOLS} index -b aligned/alignment.bam aligned/alignment.bai
	touch "done/index.done"
fi

if [ ! -f "done/statistics.done" ]; then
	rg_progress 85 "Calculating alignment statistics"
	${WD}/alignment_statistics.sh aligned/alignment.bam > aligned/alignment_statistics.out
	${SAMTOOLS} view aligned/alignment.bam | ${WD}/singlecolumnaverager.awk > aligned/fragments.out
	touch "done/statistics.done"
fi

## Genotype QC
if [ ! -f "done/genotypeqc.done" ]; then
	rg_progress 90 "Running HaplotypeCaller"
	${WD}/stdwrap.sh ${JAVA} ${JavaOptions} \
		-jar ${GATK} -T HaplotypeCaller \
		-R ${HCRef} \
		--genotyping_mode GENOTYPE_GIVEN_ALLELES \
		-L ${DBSNP} --dbsnp ${DBSNP} --alleles ${DBSNP} \
		${HaplotypeCallerOptions} \
		-I aligned/alignment.bam \
		-o aligned/qc_genotype.vcf \
		> aligned/gatk_HaplotypeCaller.out
	touch "done/genotypeqc.done"
fi

rg_progress 95 "Copying result files to project archive"
cp masked/*.out ${WD}
cp aligned/*.out ${WD}
cp aligned/alignment_picardmetrics.csv ${WD}
mkdir -p ${HisatFolder}
rm -rf ${HisatFolder}/*
cp aligned/* ${HisatFolder}
ls -1 ${HisatFolder}/* >> ${WD}/files.out

rg_progress 99 "Analysis completed, cleaning up..."
