#!/bin/bash
##
## Pipeline script for the calculating Methylation beta values. It requires a pair of IDAT
## that are either already in the project-archive directory for the Methylation item
## or in the MethylationArchive. In the latter case, the IDAT files are first copied to
## the project-archive.

##
## Environment variables that should be defined (in job.sh) before calling this script
##  -IdatFolder: Path to folder where IDAT should be found (or copied to)
##  -ScanFolder: Path to folder where output from scanner is stored (only used if there are no IDAT files in the IdatFolder)
##  -ExternalMethylationName: External name of the Methylation item that is being analyzed
##  -BeadChipId: Sentrix array ID
##  -BeadChipPosition: Position of this sample on the array
##  -TMPDIR: Path to a local temporary working directory
##

set -e

## Import utility functions
source ./reggie-utils.sh

## Verify settings in options
rg_var_isdir "TMPDIR" "WD"
rg_var_isset "MethylationName" "ExternalMethylationName" "BeadChipId" "BeadChipPosition"
rg_var_isfile "ManifestFile" "MaskFile"

## Move to the temporary working directory
cd ${TMPDIR}
mkdir -p beta
mkdir -p done

IDAT_BASE="${IdatFolder}/${MethylationName}"
RED="${IDAT_BASE}_Red.idat"
GRN="${IDAT_BASE}_Grn.idat"

## Ensure IDAT files are in the IdatFolder
if [[ ! -f "${RED}" || ! -f "${GRN}" ]]; then		
	rg_progress 10 "Copying IDAT files"
	
	## Expect IDAT files to be located in $ScanFolder/$BeadChipId_$BeadChipPosition_Red.idat
	rg_var_isdir "ScanFolder"
	RED_SRC="${ScanFolder}/${BeadChipId}_${BeadChipPosition}_Red.idat"
	GRN_SRC="${ScanFolder}/${BeadChipId}_${BeadChipPosition}_Grn.idat"
	rg_var_isfile "RED_SRC" "GRN_SRC"
	
	mkdir -p "${IdatFolder}"
	\cp "${RED_SRC}" "${RED}"
	\cp "${GRN_SRC}" "${GRN}"
fi

if [ ! -f "done/beta.done" ]; then
	## Call R script that calculates beta values and some other stuff
	${WD}/stderrwrap.sh Rscript ${WD}/beta_values.R \
		"${IDAT_BASE}" beta \
		"${ManifestFile}" "${MaskFile}" \
		"${ExternalMethylationName}" \
		"${WD}/progress" \
		3> minfi.out
	
	touch "done/beta.done"
fi

rg_progress 95 "Copying result files to project archive"
\cp beta/*.tsv.gz ${IdatFolder}
\cp beta/*.pdf ${IdatFolder}
cp beta/stats.out ${WD}/stats.out
ls -1 ${IdatFolder}/* > ${WD}/files.out

rg_progress 99 "Analysis completed, cleaning up..."
