#!/usr/bin/env Rscript

# $Id: mutation_signature.R 5807 2020-01-10 07:18:28Z nicklas $
# A simple helper script for calculating mutation signature scores
# based on COSMIC_Cancer_signatures_probabilities.RData
#
# The script need 3 or 4 parameters
# 1: Name of sample (used as table header and in the plot)
# 2: Path to VCF file with mutations
# 3: Path to COSMIC_Cancer_signatures_probabilities.RData
# 4: (Optional) Path to output pdf file
#
# The script will output scores for each of the 30 signatures as a
# tab-separated table to stdout
#

## Get command line arguments
args = commandArgs(trailingOnly=TRUE)
if (length(args) < 3) stop('At least 3 parameters are required')

sampleName <- args[1]
vcfPath <- args[2]
cosmicSignature <- args[3]

if (!file.exists(vcfPath)) stop(paste('Can\'t find VCF file: ', vcfPath, sep=''))
if (!file.exists(cosmicSignature)) stop(paste('Can\'t find COSMIC signature file: ', cosmicSignature, sep=''))

plot <- length(args) >= 4
if (plot)
{
	pdfPath = args[4]
	if (!dir.exists(dirname(pdfPath))) stop(paste('Can`t find directory for PDF file: ', pdfPath, sep=''))
}

## Other configuration options
ref_genome <- "BSgenome.Hsapiens.UCSC.hg38"

## Load libraries
library(GenomeInfoDbData)
library(MutationalPatterns)
library(BSgenome)
library(ref_genome, character.only = TRUE)

## Load the mutation signatures
load(cosmicSignature)
## reduce object matrix to signature columns only
cancer_signatures = as.matrix(cancer_signatures[,4:33])

## read vcf 
vcfs <- read_vcfs_as_granges(vcfPath, sampleName, ref_genome)

## extract mutations
mut_mat <- mut_matrix(vcf_list = vcfs, ref_genome = ref_genome)

## fit mutations to cancer signatures
fit_res <- fit_to_signatures(mut_mat, cancer_signatures)

## get contribution
x <- fit_res$contribution

## plot as pdf?
if (plot)
{
	signature.color<-c(
			rgb(85/255,46/255,141/255), #s1
			rgb(240/255,105/255,160/255), #s2
			rgb(7/255,83/255,157/255),#s3
			"lightgray",#s4
			rgb(140/255,108/255,175/255),#s5
			rgb(10/255,110/255,56/255),#s6
			"darkgray",#s7
			rgb(68/255,147/255,198/255),#s8
			"beige",#s9
			"darkslategray",#s10
			"goldenrod2",#s11
			"firebrick4",#s12
			rgb(250/255,197/255,192/255),#s13
			"azure2",#s14
			"burlywood3",#s15
			"darksalmon",#s16
			rgb(218/255,218/255,234/255),#s17
			rgb(158/255,154/255,200/255),#s18
			"darkorchid4",#s19
			rgb(66/255,171/255,93/255),#s20
			"orange",#s21
			"darkred",#s22
			"black",#s23
			"chocolate",#s24
			"cornflowerblue",#s25
			"cadetblue4",#s26
			"aquamarine4",#s27
			"aquamarine",#s28
			"antiquewhite",#s29
			"yellow" #s30
	)
	
	names(signature.color) <- paste("Signature.",1:30,sep="")
	
	main.label.file <- basename(vcfPath)
	
	####### Plot and calculate proportions
	pdf(pdfPath)
	
	prop.x <- prop.table(x, 2)
	ii <- match(rownames(prop.x), names(signature.color))
	signature.color <- signature.color[ii]
	
	#select signatures with some contribution
	select <- which(rowSums(prop.x) > 0)
	
	## plot
	my.xlim <- c(0, ncol(prop.x))
	if (ncol(prop.x)==1)
	{
		my.xlim <- c(0, 2) #increases margins on right side
		prop.x <- as.matrix(prop.x[select,]) 
		signature.color <- signature.color[select]
	}
	barplot(prop.x*100,
			xlim=my.xlim,
			ylab="%",
			main=sampleName,
			cex.main=0.7,
			las=2,
			cex.names = 0.6,
			col=signature.color,
			legend.text=rownames(prop.x),
			args.legend=list("top", bg="white", ncol=1, cex=0.7)
	)
	
	my.at <- as.numeric(filter(x=c(0,cumsum(round(prop.x[,1], 3)*100)), filter=rep(1, 2)) / 2)[1:length(prop.x[,1])]
	my.sign <- paste("s.", sapply(row.names(prop.x), function(x) strsplit(x, "\\.")[[1]][2]), sep="")
	my.pcr <- paste(round(prop.x[,1], 3)*100, "%", sep="")
	my.labels <- paste(my.pcr, " (", my.sign,")", sep="")
	axis(4, at=my.at, labels=my.labels, las=2, cex.axis=0.7, line=-12, lwd=0)
	
	mtext(paste("signature proportions:", main.label.file), line=0.5, cex=0.6)
	foo <- dev.off()
	
}# end if plot

## print table to stdout
write.table(x, file=stdout(), row.names=TRUE, col.names=FALSE, quote=FALSE, sep="\t")	
