#!/bin/bash
##
## Pipeline script for the Targeted Genotyping analysis. 
## Environment variables that should be defined (in job.sh) before calling this script
##  -BamFile: Path to alignment BAM file
##  -BaseDir: Path to directory with variant call databases
##  -JAVA: Path to java
##  -GATK: Path to the GATK JAR file
##  -VCFANNO: Path to the vcfanno program
##  -SNPEFF: Path to the SnpEff JAR file
##  -Genome: Path to the reference genome 
##  -HaplotypeCallerOptions: Options to HaplotypeCaller
##  -VcfannoOptions: Options to the vcfanno program
##  -SnpEffOptions: Options to the SnpSift program
##  -VcfFolder: Path to folder where result VCF files should be saved
##  -TMPDIR: Path to a local temporary working directory
##

set -eo pipefail

## Import utility functions
source ./reggie-utils.sh
source ./variantcall-utils.sh

## Verify settings in options
rg_var_isdir "TMPDIR" "WD" "VcfFolder" "BaseDir"
rg_var_isfile "JAVA" "GATK" "VCFANNO" "SNPEFF" "Genome" "BamFile"
rg_var_isset "HaplotypeCallerOptions" "VcfannoOptions" "SnpEffOptions"
rg_file_exists "${WD}/target_info.txt"

## Move to the temporary working directory
cd ${TMPDIR}
mkdir -p tmp
mkdir -p gatk_tmp
mkdir -p vcf
mkdir -p done

NUM_LINES=`wc -l < ${WD}/target_info.txt`
CURRENT_LINE=0
while IFS=$',' read TARGET_NAME TARGET_VCF; do
	CURRENT_LINE=$(( ${CURRENT_LINE}+1 ))
	PROGRESS=$(( 10+${CURRENT_LINE}*80/${NUM_LINES} ))
	rg_progress ${PROGRESS} "Genotyping target: ${TARGET_NAME}"
	TargetVcfPath="${BaseDir}/targeted/${TARGET_VCF}"
	rg_file_exists "${TargetVcfPath}"

	# Run HaplotypeCaller to genotype the targeted locations
	if [ ! -f "done/${TARGET_NAME}.haplotypecaller.done" ]; then
		${WD}/stdwrap.sh ${JAVA} ${JavaOptions} \
			-jar ${GATK} HaplotypeCaller \
			-R ${Genome} \
			-L ${TargetVcfPath} \
			--alleles ${TargetVcfPath} \
			--tmp-dir ${TMPDIR}/gatk_tmp \
			${HaplotypeCallerOptions} \
			-I ${BamFile} \
			-O tmp/${TARGET_VCF}.1.hc \
			>> haplotypecaller.out
		touch "done/${TARGET_NAME}.haplotypecaller.done"
	fi
		
	# Use 'LeftAlignAndTrim' to split variants on the same genomic location
	# otherwise we can't annotate properly since different alternate alleles
	# result in different amino acid changes
	if [ ! -f "done/${TARGET_NAME}.split.done" ]; then
		${WD}/stdwrap.sh ${JAVA} ${JavaOptions} \
			-jar ${GATK} LeftAlignAndTrimVariants \
			-R ${Genome} \
			--split-multi-allelics true \
			--add-output-vcf-command-line false \
			-V tmp/${TARGET_VCF}.1.hc \
			-O tmp/${TARGET_VCF}.2.split \
			>> split.out
		touch "done/${TARGET_NAME}.split.done"
	fi
		
	# Copy TYPE annotation from that target VCF file
	# We need a compressed and indexed version of the target VCF 
	# and a .toml file that define the fields to copy
	if [ ! -f "done/${TARGET_NAME}.type.done" ]; then
		bgzip -c ${TargetVcfPath} > tmp/${TARGET_VCF}.gz
		tabix tmp/${TARGET_VCF}.gz
		echo '[[annotation]]' >> tmp/${TARGET_VCF}.toml
		echo "file=\"tmp/${TARGET_VCF}.gz\"" >> tmp/${TARGET_VCF}.toml
		echo 'fields=["TYPE"]' >> tmp/${TARGET_VCF}.toml
		echo 'names=["TYPE"]' >> tmp/${TARGET_VCF}.toml
		echo 'ops=["self"]' >> tmp/${TARGET_VCF}.toml
		
		vcf_annotate tmp/${TARGET_VCF}.2.split tmp/${TARGET_VCF}.toml tmp/${TARGET_VCF}.3.type

		# Remove lines without a TYPE annotation
		# They are extra results that HaplotypeCaller creates
		# by combining nearby SNP variants.
		grep "\(^#\|TYPE\=\)" tmp/${TARGET_VCF}.3.type > tmp/${TARGET_VCF}.4.grep

		touch "done/${TARGET_NAME}.type.done"
	fi
	
	# Annotate with vcfanno
	if [ ! -f "done/${TARGET_NAME}.vcfanno.done" ]; then
		vcf_annotate tmp/${TARGET_VCF}.4.grep "${VcfannoOptions}" tmp/${TARGET_VCF}.5.ann
		touch "done/${TARGET_NAME}.vcfanno.done"
	fi
	
	# Annotate with snpEff
	if [ ! -f "done/${TARGET_NAME}.snpeff.done" ]; then
		snpeff_annotate tmp/${TARGET_VCF}.5.ann "${SnpEffOptions}" tmp/${TARGET_VCF}.6.eff
		touch "done/${TARGET_NAME}.snpeff.done"
	fi
	
	# Remove '##contig' entries from the final VCF since they are not really needed
	if [ ! -f "done/${TARGET_NAME}.done" ]; then
		sed '/##contig/d' tmp/${TARGET_VCF}.6.eff > vcf/genotype_${TARGET_VCF}
		echo "[${TARGET_NAME}]" >> ${WD}/files.out
		echo "genotype_${TARGET_VCF}" >> ${WD}/files.out
		touch "done/${TARGET_NAME}.done"
	fi
	
done < ${WD}/target_info.txt

rg_progress 90 "Copying result files to project archive"
\cp vcf/* ${VcfFolder}

rg_progress 99 "Analysis completed, cleaning up..."
